/*
    chips.c - Part of sensors, a user-space program for hardware monitoring
    Copyright (c) 1998, 1999, 2001  Frodo Looijaard <frodol@dds.nl>
    and Mark D. Studebaker <mdsxyz123@yahoo.com>
 
    chips.c - It has been modified to be used on HotApplet by Daniel
    Baeyens <daniel.baeyens@hispalinux.es>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include "chips.h"
#include <sensors/sensors.h>
#include <sensors/chips.h>

#include <gnome.h>

int get_feature(sensors_chip_name *, gchar *, gchar *);

int sensors_get_label_and_valid(sensors_chip_name name, int feature, char **label, int *valid)
{
    int err;
    err = sensors_get_label(name,feature,label);
    if (!err)
      err = sensors_get_ignored(name,feature);
    
    if (err >= 0) {
         *valid = err;
         err = 0;
    }
    return err;
}

int get_feature(sensors_chip_name *chip, gchar *feature, gchar *r_string)
{
  gchar *cadena;
  cadena = g_malloc0(7*sizeof(gchar));	
  
  if (!strcmp(chip->prefix,"via686a"))
  {

     if (!get_VIA686A_features(*chip,feature, cadena))
     {

         g_snprintf(r_string, 20, "%s \302\260C", cadena);
	 return 0;

     }
     else
     {

         g_snprintf(r_string, 20, "%s \302\260C", cadena);
	 return 1;

     }
  }
  else if (!strcmp(chip->prefix, "w83781d") ||
	   !strcmp(chip->prefix, "w83782d") ||
	   !strcmp(chip->prefix, "w83783s") ||
	   !strcmp(chip->prefix, "w83627hf") ||
	   !strcmp(chip->prefix, "w83697hf") ||
	   !strcmp(chip->prefix, "as99127f")
	   )
  {

     if (!get_W83781D_features(*chip,feature, cadena))
     {
	
         g_snprintf(r_string, 20, "%s \302\260C", cadena);
	 
	 return 0;

     }
     else
     {

         g_snprintf(r_string, 20, "%s \302\260C", cadena);
	 return 1;

     }
  } else if (!strcmp(chip->prefix, "it87")){
  
     if (!get_IT87_features(*chip,feature, cadena))
     {
         g_snprintf(r_string, 20, "%s \302\260C", cadena);
	 return 0;
     }
     else
     {
         g_snprintf(r_string, 20, "%s \302\260C", cadena);
	 return 1;
     }
  }

  g_snprintf(r_string, 20, "%s \302\260C", cadena);
  return 0;
}
	
