/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  HotApplet was writen by Daniel Baeyens <daniel.baeyens@hispalinux.es>
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sensors/sensors.h>
#include <sensors/error.h>
#include <sensors/chips.h>

#include <glade/glade.h>

#include <panel-applet.h>
#include <panel-applet-gconf.h>
#include <gnome.h>

#include "hotapplet.h"
#include "chips.h"

void preferences_cb(BonoboUIComponent *uic,
		    HotApplet         *hottie,
		    const char        *verbname);


void about_cb(BonoboUIComponent *uic,
	      HotApplet         *hottie,
	      const char        *verbname);

static gboolean timeout_cb(HotApplet *hottie);
	
static const BonoboUIVerb HotApplet_verbs[] = {
  BONOBO_UI_UNSAFE_VERB ("Preferences", preferences_cb),
  BONOBO_UI_UNSAFE_VERB ("About",about_cb),
  BONOBO_UI_VERB_END
};

static GladeXML *xml = NULL;

void buttonclose_pressed(GtkButton *button,
      		         gpointer data){
	gtk_widget_hide(data);
}

void timeoutspin_changed(GtkButton *button,
		         HotApplet *data){
	gint new_time;

	new_time = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(button)) * 1000;

	if (new_time != data->timeout){
	     data->timeout = new_time;
	     gtk_timeout_remove(data->timeout_id);
	     data->timeout_id = gtk_timeout_add(new_time,
			                        (GtkFunction) timeout_cb,
						data);
	}
}

void combosensors_changed(GtkWidget *widget,
		          HotApplet *data){
  if (!strcmp(gtk_entry_get_text(GTK_ENTRY(widget)), "Temperature 1")){
     panel_applet_gconf_set_string(data->applet, "state", "TEMP1", NULL);
  } else if (!strcmp(gtk_entry_get_text(GTK_ENTRY(widget)), "Temperature 2")){
     panel_applet_gconf_set_string(data->applet, "state", "TEMP2", NULL);
  } else if (!strcmp(gtk_entry_get_text(GTK_ENTRY(widget)), "Temperature 3")){
     panel_applet_gconf_set_string(data->applet, "state", "TEMP3", NULL);
  }
}

void preferences_cb(BonoboUIComponent *uic,
		    HotApplet         *hottie,
		    const char        *verbname){
        static GtkWidget *global_property_box = NULL,
                         *glade_property_box = NULL;
	
	GList *items = NULL;
	
        GtkWidget *TimeoutSpin = NULL,
                  *ButtonClose = NULL,
		  *Combosensors = NULL,
		  *Comboentry = NULL;

        g_return_if_fail (PANEL_IS_APPLET (PANEL_APPLET (hottie->applet)));

        if (hottie->prefs != NULL)
        {
                gtk_widget_show (hottie->prefs);
                gtk_window_present (GTK_WINDOW (hottie->prefs));
                return;
        }

        if (global_property_box == NULL) {
                xml = glade_xml_new (HOTAPPLET_GLADEDIR"/GNOME_HotApplet.glade", NULL, NULL);
                glade_property_box = glade_xml_get_widget (xml,"PrefWindow");
        }
 
        hottie->prefs = GTK_WIDGET(glade_property_box);

        TimeoutSpin = glade_xml_get_widget (xml, "TimeoutSpin");
        ButtonClose = glade_xml_get_widget (xml, "ButtonClose");
	Combosensors = glade_xml_get_widget(xml, "combo-sensors");
	Comboentry = glade_xml_get_widget(xml, "combolist-sensors");

	items = g_list_append(items, "Temperature 1");
	items = g_list_append(items, "Temperature 2");
	items = g_list_append(items, "Temperature 3");

	gtk_combo_set_popdown_strings (GTK_COMBO (Combosensors), items);
	
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(TimeoutSpin),
			          hottie->timeout/1000);
	
	if (hottie->chip->prefix != NULL){
	g_signal_connect(GTK_OBJECT(TimeoutSpin),
			"value-changed",
			G_CALLBACK (timeoutspin_changed),
			hottie);

	g_signal_connect (G_OBJECT (GTK_COMBO(Combosensors)->entry),
			  "changed",
			  G_CALLBACK (combosensors_changed),
			  hottie);
	}
	
	g_signal_connect(GTK_OBJECT(ButtonClose),
			 "clicked",
			 G_CALLBACK (buttonclose_pressed),
			 hottie->prefs);

        gtk_widget_show_all (hottie->prefs);
};

void about_cb(BonoboUIComponent *uic,
	      HotApplet         *hottie,
	      const char        *verbname){
	GdkPixbuf *iconapplet;
        const char *authors[] = {"Daniel Baeyens <daniel.baeyens@hispalinux.es>", "MArk Finlay <sisob@eircom.net>", NULL};
	                                                                                                                              
	/* Feel free to put your names here translators :-) */
//	char *translators = _("TRANSLATORS");
	                                                                                                                                
	if (hottie->about)
	{
	     gtk_window_present(GTK_WINDOW(hottie->about));
	     return;
	}
	
	iconapplet = gdk_pixbuf_new_from_file(GNOME_ICONDIR"/hotapplet/hotapplet.png",NULL);
	
	hottie->about = GTK_WIDGET(gnome_about_new ("HotApplet",
					            VERSION,
						    "Copyright 2003 Daniel Baeyens",
	 	                                    "This applet shows motherboard sensors values as CPU Temperature, CPU fan rpm's, voltages...",
		                                   (const char **) authors,
						   NULL,
						   NULL,
				                   iconapplet));
						  
	gdk_pixbuf_unref(iconapplet);
	g_object_add_weak_pointer(G_OBJECT(hottie->about), (gpointer*)&hottie->about);
	gtk_widget_show(GTK_WIDGET(hottie->about));
};


static void
set_appearance(HotApplet *hottie, gchar *feature)
{
  gchar *for_tooltip;
  gchar *for_tooltip_temp;
  gchar *label;
  
  label = g_new0(gchar,15);
  for_tooltip = g_new0(gchar, 100);
  for_tooltip_temp = g_new0(gchar,100);
  
  if (strcmp(hottie->chip->prefix, "it87")){
    if (!strcmp("TEMP1", feature))
      {
	gtk_image_set_from_file(GTK_IMAGE(hottie -> pix),
				GNOME_ICONDIR"/hotapplet/cpu.png");
	
	g_snprintf(for_tooltip,
	           90,
	           "Sensor model: \t%s\n\nTemperature 1",
	   	   hottie->chip->prefix);
	
	if (!get_feature((sensors_chip_name *)hottie->chip,
			 "HYST1",
			 label))
	  {
	    g_snprintf(for_tooltip_temp, 
			    90,
			    "%s\nHysteresis: \t%s",
			    for_tooltip,
			    label);
	    if (!get_feature((sensors_chip_name *)hottie->chip,
			     "OVER1",
			     label))
	      {
		
		g_snprintf(for_tooltip,
		 	   90,
			   "%s\nLimit: \t\t%s\0",
			   for_tooltip_temp,
			   label);
	      }
	    
	  }
	
	gtk_tooltips_set_tip(hottie->tooltip, GTK_WIDGET(hottie->applet), for_tooltip, "");
	
      }

   else if (!strcmp("TEMP2", feature))
      {
	gtk_image_set_from_file(GTK_IMAGE(hottie -> pix),
				GNOME_ICONDIR"/hotapplet/cpu.png");
	
	for_tooltip = g_strdup_printf("Sensor model: \t%s\n\nTemperature 2",
				      hottie->chip->prefix);
	
	if (!get_feature((sensors_chip_name *)hottie->chip,
			 "HYST2",
			 label))
	  {
	    for_tooltip_temp = g_strdup_printf("%s\nHysteresis: \t%s",
					       for_tooltip,
					       label);
	    if (!get_feature((sensors_chip_name *)hottie->chip,
			     "OVER2",
			     label))
	      {
		
		for_tooltip = g_strdup_printf("%s\nLimit: \t\t%s\0",
					      for_tooltip_temp,
					      label);
		
	      }
	    
	  }
	
	gtk_tooltips_set_tip(hottie->tooltip, GTK_WIDGET(hottie->applet), for_tooltip, "");
      }
  else if (!strcmp("TEMP3", feature))
      {
	gtk_image_set_from_file(GTK_IMAGE(hottie -> pix),
				GNOME_ICONDIR"/hotapplet/cpu.png");
	
	for_tooltip = g_strdup_printf("Sensor model: \t%s\n\nTemperature 3",
				      hottie->chip->prefix);
	
	if (!get_feature((sensors_chip_name *)hottie->chip,
			 "HYST3",
			 label))
	  {
	    for_tooltip_temp = g_strdup_printf("%s\nHysteresis: \t%s",
					       for_tooltip,
					       label);
	    if (!get_feature((sensors_chip_name *)hottie->chip,
			     "OVER3",
			     label))
	      {
		
		for_tooltip = g_strdup_printf("%s\nLimit: \t\t%s\0",
					      for_tooltip_temp,
					      label);
		
	      }
	    
	  }
	
	gtk_tooltips_set_tip(hottie->tooltip, GTK_WIDGET(hottie->applet), for_tooltip, "");
      }
  else    
      {
	gtk_tooltips_set_tip(hottie->tooltip, GTK_WIDGET(hottie->applet), "error", "");
      }  
  } else {
	gtk_image_set_from_file(GTK_IMAGE(hottie -> pix),
				GNOME_ICONDIR"/hotapplet/cpu.png");
			
	for_tooltip = g_strdup_printf("Sensor model: \t%s",
				      hottie->chip->prefix);
  }
  
  g_free(for_tooltip);
  g_free(for_tooltip_temp);
  g_free(label);
  
}

static gboolean
timeout_cb(HotApplet *hottie)
{
  double valor;
  gchar buffer[7];
  gchar *cadena;
  gchar *label;

  label = g_malloc0(sizeof(gchar) * 10);
  cadena = g_malloc0(sizeof(gchar) * 10);
 
  if (PANEL_IS_APPLET(hottie->applet)){
    cadena = panel_applet_gconf_get_string(hottie->applet, "state", NULL);
    
    if (cadena == NULL){
        panel_applet_gconf_set_string(hottie->applet, "state", "TEMP1", NULL);
	cadena = panel_applet_gconf_get_string(hottie->applet, "state", NULL);
    }
    
    if (!get_feature((sensors_chip_name *)hottie->chip,
		      cadena,
		      label))
    {
      set_appearance(hottie,
		     cadena);
      gtk_label_set_text(GTK_LABEL(hottie->hotlabel),
			 label);
    }
  
//  gtk_widget_set_size_request(GTK_WIDGET(hottie->hbox), -1, 100);
  
  }
  gtk_widget_show_all (GTK_WIDGET (hottie->applet));
 
  g_free(label);
  
  return TRUE;
}

static void
do_sensor_detection(HotApplet *hottie)
{
    sensors_chip_name *mask = malloc(sizeof(sensors_chip_name));
  
  int chip_nr;
  gchar *tooltip_text;
  gint test = 0;
  sensors_chip_name *testchip = g_malloc0(sizeof(sensors_chip_name));
  
  tooltip_text = g_new0(gchar,100);

  chip_nr = 0;

  testchip = (sensors_chip_name *)sensors_get_detected_chips(&chip_nr);
  for (test = 0; testchip != NULL; test++)
    {
      
      testchip = (sensors_chip_name *)sensors_get_detected_chips(&chip_nr);
    }
  
  chip_nr = 0;
  
  mask->prefix = SENSORS_CHIP_NAME_PREFIX_ANY;
  mask->bus = SENSORS_CHIP_NAME_BUS_ANY;
  mask->addr = SENSORS_CHIP_NAME_ADDR_ANY;

  hottie->chip = (sensors_chip_name *)sensors_get_detected_chips(&chip_nr);
  
  if (hottie->chip == NULL){
    hottie->pix = GTK_WIDGET(gtk_image_new_from_file (GNOME_ICONDIR"/hotapplet/warning.png"));
    tooltip_text = g_strdup_printf("No sensors found on\nyour system!");
    gtk_tooltips_set_tip(hottie->tooltip, GTK_WIDGET(hottie->applet), tooltip_text, "");
  } else {
    if (sensors_match_chip(*hottie->chip, *mask))
      {
	hottie->timeout = 3000;
	hottie->timeout_id = gtk_timeout_add(3000, (GtkFunction) timeout_cb, hottie);
      }
  }
  gtk_widget_show_all(GTK_WIDGET(hottie->applet));
  g_free (tooltip_text);

}

static gint
open_config(HotApplet *hottie)
{
  hottie->config_file = fopen("/etc/sensors.conf", "r");
  if (! hottie->config_file)
    {
      return 0;
    }
  
  return 1;
}

static void destroyed_cb(GtkWidget *widget,
		         gpointer data){
      FILE *id_lock;
      int instances;

      id_lock = fopen("/tmp/.hotapplet.0.LOCK", "r");
      fscanf(id_lock, "%d", &instances);
      fclose (id_lock);
      if (instances > 1){
	 instances--;
         id_lock = fopen("/tmp/.hotapplet.0.LOCK", "w");
         fprintf(id_lock, "%d\n%d", instances, getpid());
         fclose(id_lock);
      } else {     
         unlink("/tmp/.hotapplet.0.LOCK");
      }
}

static void 
hot_applet_fill(PanelApplet *applet)
{
  HotApplet *hottie = g_new0(HotApplet,1);
  gchar *tooltip_text = NULL;
  gint res = 0;
  
  int instances;
  int pid;
  FILE *id_lock;

  hottie->prefs=NULL;
  
  hottie->chip = g_malloc0(sizeof(sensors_chip_name));
  
  hottie->applet = applet;
  
  panel_applet_add_preferences(applet, "/schemas/apps/hotapplet/prefs", NULL);
  
  hottie->pix = GTK_WIDGET(gtk_image_new_from_file(GNOME_ICONDIR"/hotapplet/warning.png"));
  
  hottie->hotlabel = gtk_label_new("-- \302\260C");
                                   
  hottie->hbox = GTK_WIDGET(gtk_hbox_new(FALSE,2));
  
  gtk_container_add(GTK_CONTAINER (hottie->hbox), GTK_WIDGET(hottie->pix));
  gtk_container_add(GTK_CONTAINER (hottie->hbox), GTK_WIDGET(hottie->hotlabel));
  gtk_container_add(GTK_CONTAINER (hottie->applet), GTK_WIDGET(hottie->hbox));
  
  hottie->tooltip = gtk_tooltips_new();
  gtk_tooltips_enable(hottie->tooltip);

  panel_applet_setup_menu_from_file (PANEL_APPLET(hottie->applet),
                                     UIDIR,
                                     "GNOME_HotApplet.xml",
                                     NULL,
                                     HotApplet_verbs,
                                     hottie);
  
  g_signal_connect(hottie->hbox,
		   "destroy",
		   GTK_SIGNAL_FUNC(destroyed_cb),
		   hottie);
  
  if (open_config(hottie))
    {		
      id_lock = fopen("/tmp/.hotapplet.0.LOCK", "r");
      if (!id_lock){
          res = sensors_init(hottie->config_file);
	  id_lock = fopen("/tmp/.hotapplet.0.LOCK", "w");
	  fprintf(id_lock, "%d\n%d\n", 1, getpid());
	  fclose(id_lock);
      }else{
	  fscanf(id_lock, "%d\n%d\n", &instances, &pid);
	  if (pid == getpid()){
	     instances++;
             fclose (id_lock);
	     id_lock = fopen("/tmp/.hotapplet.0.LOCK", "w");
	     fprintf(id_lock, "%d\n%d\n", instances, getpid());
	     fclose(id_lock);
	  } else {
	     unlink ("/tmp/.hotapplet.0.LOCK");
             res = sensors_init(hottie->config_file);
	     id_lock = fopen("/tmp/.hotapplet.0.LOCK", "w");
	     fprintf(id_lock, "%d\n%d\n", 1, getpid());
	     fclose(id_lock);
	  }
      }
      
      if(res == 0)
	{
	  if (res == -SENSORS_ERR_PROC)
	    {
	      hottie->pix = GTK_WIDGET(gtk_image_new_from_file (GNOME_ICONDIR"/hotapplet/warning.png"));
	      tooltip_text = g_strdup_printf("No /proc/ entries. Make sure\nyou have i2c-proc module loaded");
	      gtk_tooltips_set_tip(hottie->tooltip, GTK_WIDGET(hottie->applet), tooltip_text, "");
	    } else
	      {
		tooltip_text = g_strdup_printf("Detecting sensors...");	 
		do_sensor_detection(hottie);
	      }
	}
      else
	{
	  hottie->pix = GTK_WIDGET(gtk_image_new_from_file (GNOME_ICONDIR"/hotapplet/warning.png"));
	  g_snprintf(tooltip_text, 20, "No sensors found");
	  gtk_tooltips_set_tip(hottie->tooltip, GTK_WIDGET(hottie->applet), tooltip_text, "");
	}
    }
  else
    {
      hottie->pix = GTK_WIDGET(gtk_image_new_from_file (GNOME_ICONDIR"/hotapplet/warning.png"));
      g_snprintf(tooltip_text, 20, "Can't init sensors");
      gtk_tooltips_set_tip(hottie->tooltip, GTK_WIDGET(hottie->applet), tooltip_text, "");
    }
  
  
  gtk_widget_set_size_request(GTK_WIDGET(hottie->hbox), 70, -1);
  
  gtk_widget_show_all(GTK_WIDGET(hottie->applet));
  g_free (tooltip_text);
  
}

static gboolean
hot_applet_start (PanelApplet *applet,
		  const gchar *iid,
		  gpointer     data)
{
  if (strcmp (iid, "OAFIID:GNOME_HotApplet") != 0)
    return FALSE;
  
  hot_applet_fill(applet);
  
  return TRUE;
}


PANEL_APPLET_BONOBO_FACTORY ("OAFIID:GNOME_HotApplet_Factory",
			     PANEL_TYPE_APPLET,
			     "Hot Applet",
			     "0",
			     hot_applet_start,
			     NULL);

